﻿/// @file       vcsharp_pwm_demo
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $ guc $
/// @date       $ 4/24/14 $
/// @brief      This is a Visual C# demo code to demonstrate how to use Toradex PWM Library in C#

using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;                              ///< To use Toradex C Win32 DLLs

namespace vcsharp_pwm_demo
{
    public partial class Form1 : Form
    {
        public class Constant                                      ///< Class for the Constants used
        {
            public const Int32 PWM_A = 59;                         ///< PWM_A at SODIMM pin 59
            public const Int32 PWM_B = 28;                         ///< PWM_B at SODIMM pin 28
            public const Int32 PWM_C = 30;                         ///< PWM_C at SODIMM pin 30
            public const Int32 PWM_D = 67;                         ///< PWM_D at SODIMM pin 67
            public const Int32 TYPE_PXA270 = 17;
            public const Int32 TYPE_PXA300 = 8;
            public const Int32 TYPE_PXA310 = 9;
            public const Int32 TYPE_PXA320 = 2;
            public const Int32 TYPE_TEGRA2 = 68287497;
            public const Int32 TYPE_TEGRA3 = 68353033;
            /// Prescale values to obtain required frequencies at fixed period of 256 for Tegra and 1024 for PXA
            public const int TEGRA2_PRESCALE_50Hz = 3774; 
            public const int TEGRA2_PRESCALE_100Hz = 1887;
            public const int TEGRA2_PRESCALE_500Hz = 377;
            public const int TEGRA2_PRESCALE_1kHz = 189;
            public const int TEGRA2_PRESCALE_2kHz = 94;
            public const int TEGRA2_PRESCALE_5kHz = 38;
            public const int TEGRA3_PRESCALE_50Hz = 7042;
            public const int TEGRA3_PRESCALE_100Hz = 3521;
            public const int TEGRA3_PRESCALE_500Hz = 704;
            public const int TEGRA3_PRESCALE_1kHz = 352;
            public const int TEGRA3_PRESCALE_2kHz = 176;
            public const int TEGRA3_PRESCALE_5kHz = 70;
            public const int PXA_PRESCALE_200Hz = 64;
            public const int PXA_PRESCALE_250Hz = 51;
            public const int PXA_PRESCALE_500Hz = 25;
            public const int PXA_PRESCALE_1kHz = 13;
            public const int PXA_PRESCALE_2kHz = 6;
            public const int PXA_PRESCALE_5kHz = 3;
            public const int TEGRA_PERIOD = 256;
            public const int PXA_PERIOD = 1024;
        }
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]                                ///< Importing DLL
        public static extern bool SetPinAltFn(Int32 pinNum, Int32 altFn, bool dirOut);    ///< External function declaration
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern bool SetGPIOAltFn(Int32 gpioNum, Int32 altFn, bool dirOut);
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void InitGPIOLib();
        [DllImport("GPIOLib.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitGPIOLib();
        [DllImport("PWMLib.dll", CharSet = CharSet.Auto)]
        public static extern void InitPWM(Int32 pwm, Int32 prescale, Int32 period);
        [DllImport("PWMLib.dll", CharSet = CharSet.Auto)]
        public static extern void SetPWMDutyPercentage(Int32 pwm, Int32 duty);
        [DllImport("PWMLib.dll", CharSet = CharSet.Auto)]
        public static extern void DeInitPWM(Int32 pwm);
        [DllImport("CoProcLib.dll", CharSet = CharSet.Auto)]
        public static extern Int32 getPROCID();

        public Int16 count = 0;
        public Int32 procID = 0;
        public Int32 dutyCycleA = 0;
        public Int32 dutyCycleB = 0;
        public Int32 dutyCycleC = 0;
        public Int32 dutyCycleD = 0;
        public Int32 channelA = 0;
        public Int32 channelB = 1;
        public Int32 channelC = 2;
        public Int32 channelD = 3;
        public Int32 altFunctionA = 0;
        public Int32 altFunctionB = 0;
        public Int32 altFunctionC = 0;
        public Int32 altFunctionD = 0;
        public Int32 prescale_1 = 0;
        public Int32 prescale_2 = 0;
        public Int32 prescale_3 = 0;
        public Int32 prescale_4 = 0;
        public Int32 prescale_5 = 0;
        public Int32 prescale_6 = 0;


        string[] tegraFrequencyArray = new string[] {"50Hz", "100Hz", "500Hz", "1kHz", "2kHz", "5kHz", "NONE"};
        string[] pxaFrequencyArray = new string[] {"200Hz", "250Hz", "500Hz", "1kHz", "2kHz", "5kHz", "NONE"};
        string dutyCycleString1 = null;
        string dutyCycleString2 = null;
        string dutyCycleString3 = null;
        string dutyCycleString4 = null;

        public Form1()
        {
            InitializeComponent();
            procID = getPROCID();                      ///< Get processor ID
            for (count = 0; count < 7; count++)        ///< Add items to combo box
            {
                if (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA3)
                {
                    comboBoxChannel1.Items.Add(tegraFrequencyArray[count]);
                    comboBoxChannel2.Items.Add(tegraFrequencyArray[count]);
                    comboBoxChannel3.Items.Add(tegraFrequencyArray[count]);
                    comboBoxChannel4.Items.Add(tegraFrequencyArray[count]);
                }
                else if (procID == Constant.TYPE_PXA270 || procID == Constant.TYPE_PXA300 || procID == Constant.TYPE_PXA310 || procID == Constant.TYPE_PXA320)
                {
                    comboBoxChannel1.Items.Add(pxaFrequencyArray[count]);
                    comboBoxChannel2.Items.Add(pxaFrequencyArray[count]);
                    comboBoxChannel3.Items.Add(pxaFrequencyArray[count]);
                    comboBoxChannel4.Items.Add(pxaFrequencyArray[count]);
                }
            }
            /// Default selected item 
            comboBoxChannel1.SelectedIndex = 6;
            comboBoxChannel2.SelectedIndex = 6;
            comboBoxChannel3.SelectedIndex = 6;
            comboBoxChannel4.SelectedIndex = 6;

            if (procID == Constant.TYPE_PXA270)    ///< PXA270
            {
                channelA = 3;
                channelB = 2;
                channelC = 0;
                channelD = 1;
                altFunctionA = 2;
                altFunctionB = 2;
                altFunctionC = 2;
                altFunctionD = 2;
                prescale_1 = Constant.PXA_PRESCALE_200Hz;
                prescale_2 = Constant.PXA_PRESCALE_250Hz;
                prescale_3 = Constant.PXA_PRESCALE_500Hz;
                prescale_4 = Constant.PXA_PRESCALE_1kHz;
                prescale_5 = Constant.PXA_PRESCALE_2kHz;
                prescale_6 = Constant.PXA_PRESCALE_5kHz;

            }
            else if (procID == Constant.TYPE_PXA300 || procID == Constant.TYPE_PXA310 || procID == Constant.TYPE_PXA320)    ///< PXA3xx
            {
                channelA = 3;
                channelB = 2;
                channelC = 0;
                channelD = 1;
                altFunctionA = 1;
                altFunctionB = 1;
                altFunctionC = 1;
                altFunctionD = 1;
                prescale_1 = Constant.PXA_PRESCALE_200Hz;
                prescale_2 = Constant.PXA_PRESCALE_250Hz;
                prescale_3 = Constant.PXA_PRESCALE_500Hz;
                prescale_4 = Constant.PXA_PRESCALE_1kHz;
                prescale_5 = Constant.PXA_PRESCALE_2kHz;
                prescale_6 = Constant.PXA_PRESCALE_5kHz;
            }
            else if (procID == Constant.TYPE_TEGRA2)    ///< T20
            {
                channelA = 0;
                channelB = 1;
                channelC = 2;
                channelD = 3;
                altFunctionA = 0;
                altFunctionB = 0;
                altFunctionC = 1;
                altFunctionD = 1;
                prescale_1 = Constant.TEGRA2_PRESCALE_50Hz;
                prescale_2 = Constant.TEGRA2_PRESCALE_100Hz;
                prescale_3 = Constant.TEGRA2_PRESCALE_500Hz;
                prescale_4 = Constant.TEGRA2_PRESCALE_1kHz;
                prescale_5 = Constant.TEGRA2_PRESCALE_2kHz;
                prescale_6 = Constant.TEGRA2_PRESCALE_5kHz;
            }
            else if (procID == Constant.TYPE_TEGRA3)    ///< T30
            {
                channelA = 0;
                channelB = 1;
                channelC = 2;
                channelD = 3;
                altFunctionA = 1;
                altFunctionB = 1;
                altFunctionC = 1;
                altFunctionD = 1;
                prescale_1 = Constant.TEGRA3_PRESCALE_50Hz;
                prescale_2 = Constant.TEGRA3_PRESCALE_100Hz;
                prescale_3 = Constant.TEGRA3_PRESCALE_500Hz;
                prescale_4 = Constant.TEGRA3_PRESCALE_1kHz;
                prescale_5 = Constant.TEGRA3_PRESCALE_2kHz;
                prescale_6 = Constant.TEGRA3_PRESCALE_5kHz;
            }
            /// Set Pin functionality as PWM and set to Output
            SetPinAltFn(Constant.PWM_A, altFunctionA, true);
            SetPinAltFn(Constant.PWM_B, altFunctionB, true);
            SetPinAltFn(Constant.PWM_C, altFunctionC, true);
            if (procID == Constant.TYPE_PXA300 || procID == Constant.TYPE_PXA310)
            {
                SetGPIOAltFn(18, altFunctionD, true);                             ///< GPIO18 as PWM at SODIMM pin 152
                SetGPIOAltFn(126, 7, true);                                       ///< Tri-state the multiplexed GPIO126
            }
            else if (procID == Constant.TYPE_PXA320)
            {
                SetGPIOAltFn(12, altFunctionD, true);                             ///< GPIO12 as PWM at SODIMM pin 152
                SetGPIOAltFn(0 + 128, 7, true);                                   ///< Tri-state the multiplexed GPIO0_2
            }
            else                                                                  ///< T20, T30 and PXA270
            {
                SetPinAltFn(Constant.PWM_D, altFunctionD, true);
            }
        }

        private void button1_Click(object sender, EventArgs e)                    ///< PWM_A Set button Clicked 
        {
            switch (comboBoxChannel1.SelectedIndex)
            {
                case 0:                                                           ///< If 1st frequency is selected from combo box
                    dutyCycleA = trackBar1.Value;                                 ///< Read Duty-cycle selected
                    InitPWM(channelA, prescale_1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);   ///< Initialise PWM Controller 
                    SetPWMDutyPercentage(channelA, dutyCycleA);                   ///< Assign percentage duty cycle in PWM Controller 
                    break;
                case 1:                                                           ///< If 2nd frequency is selected
                    dutyCycleA = trackBar1.Value;
                    InitPWM(channelA, prescale_2, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelA, dutyCycleA);
                    break;
                case 2:                                                           ///< If 3rd frequency is selected
                    dutyCycleA = trackBar1.Value;
                    InitPWM(channelA, prescale_3, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelA, dutyCycleA);
                    break;
                case 3:                                                           ///< If 4th frequency is selected
                    dutyCycleA = trackBar1.Value;
                    InitPWM(channelA, prescale_4, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelA, dutyCycleA);
                    break;
                case 4:                                                           ///< If 5th frequency is selected
                    dutyCycleA = trackBar1.Value;
                    InitPWM(channelA, prescale_5, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelA, dutyCycleA);
                    break;
                case 5:                                                           ///< If 6th frequency is selected
                    dutyCycleA = trackBar1.Value;
                    InitPWM(channelA, prescale_6, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelA, dutyCycleA);
                    break;
                default:
                    dutyCycleA = 0;
                    InitPWM(channelA, 1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelA, dutyCycleA);
                    break;
            }
        }

        private void button2_Click(object sender, EventArgs e)                    ///< PWM_B Set button Clicked
        {
            switch (comboBoxChannel2.SelectedIndex)
            {
                case 0:
                    dutyCycleB = trackBar2.Value;
                    InitPWM(channelB, prescale_1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ? Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);   ///< Initialise PWM Controller
                    SetPWMDutyPercentage(channelB, dutyCycleB);
                    break;
                case 1:
                    dutyCycleB = trackBar2.Value;
                    InitPWM(channelB, prescale_2, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelB, dutyCycleB);
                    break;
                case 2:
                    dutyCycleB = trackBar2.Value;
                    InitPWM(channelB, prescale_3, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelB, dutyCycleB);
                    break;
                case 3:
                    dutyCycleB = trackBar2.Value;
                    InitPWM(channelB, prescale_4, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelB, dutyCycleB);
                    break;
                case 4:
                    dutyCycleB = trackBar2.Value;
                    InitPWM(channelB, prescale_5, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelB, dutyCycleB);
                    break;
                case 5:
                    dutyCycleB = trackBar2.Value;
                    InitPWM(channelB, prescale_6, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelB, dutyCycleB); 
                    break;
                default:
                    dutyCycleB = 0;
                    InitPWM(channelB, 1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelB, dutyCycleB);
                    break;
            }
        }

        private void button3_Click(object sender, EventArgs e)                     ///< PWM_C Set button Clicked
        {
            switch (comboBoxChannel3.SelectedIndex)
            {
                case 0:
                    dutyCycleC = trackBar3.Value;
                    InitPWM(channelC, prescale_1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ? Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);    ///< Initialise PWM Controller
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
                case 1:
                    dutyCycleC = trackBar3.Value;
                    InitPWM(channelC, prescale_2, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
                case 2:
                    dutyCycleC = trackBar3.Value;
                    InitPWM(channelC, prescale_3, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
                case 3:
                    dutyCycleC = trackBar3.Value;
                    InitPWM(channelC, prescale_4, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
                case 4:
                    dutyCycleC = trackBar3.Value;
                    InitPWM(channelC, prescale_5, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
                case 5:
                    dutyCycleC = trackBar3.Value;
                    InitPWM(channelC, prescale_6, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
                default:
                    dutyCycleC = 0;
                    InitPWM(channelC, 1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelC, dutyCycleC);
                    break;
            }
        }

        private void button4_Click(object sender, EventArgs e)                        ///< PWM_D Set button Clicked
        {
            switch (comboBoxChannel4.SelectedIndex)
            {
                case 0:
                    dutyCycleD = trackBar4.Value;
                    InitPWM(channelD, prescale_1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ? Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);   ///< Initialise PWM Controller
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
                case 1:
                    dutyCycleD = trackBar4.Value;
                    InitPWM(channelD, prescale_2, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
                case 2:
                    dutyCycleD = trackBar4.Value;
                    InitPWM(channelD, prescale_3, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
                case 3:
                    dutyCycleD = trackBar4.Value;
                    InitPWM(channelD, prescale_4, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
                case 4:
                    dutyCycleD = trackBar4.Value;
                    InitPWM(channelD, prescale_5, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
                case 5:
                    dutyCycleD = trackBar4.Value;
                    InitPWM(channelD, prescale_6, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
                default:
                    dutyCycleD = 0;
                    InitPWM(channelD, 1, (procID == Constant.TYPE_TEGRA2 || procID == Constant.TYPE_TEGRA2) ?  Constant.TEGRA_PERIOD : Constant.PXA_PERIOD);
                    SetPWMDutyPercentage(channelD, dutyCycleD);
                    break;
            }
        }

        private void Channel1NewDutyCycle(object sender, EventArgs e)
        {
            dutyCycleA = trackBar1.Value;
            dutyCycleString1 = Convert.ToString(dutyCycleA);
            lblDutyCycle1.Text = dutyCycleString1;                                      ///< Display the Duty-cycle Selected For PWM_A
        }

        private void Channel2NewDutyCycle(object sender, EventArgs e)
        {
            dutyCycleB = trackBar2.Value;
            dutyCycleString2 = Convert.ToString(dutyCycleB);
            lblDutyCycle2.Text = dutyCycleString2;                                      ///< Display the Duty-cycle Selected For PWM_B
        }

        private void Channel3NewDutyCycle(object sender, EventArgs e)
        {
            dutyCycleC = trackBar3.Value;
            dutyCycleString3 = Convert.ToString(dutyCycleC);
            lblDutyCycle3.Text = dutyCycleString3;                                      ///< Display the Duty-cycle Selected For PWM_C
        }

        private void Channel4NewDutyCycle(object sender, EventArgs e)
        {
            dutyCycleD = trackBar4.Value;
            dutyCycleString4 = Convert.ToString(dutyCycleD);
            lblDutyCycle4.Text = dutyCycleString4;                                      ///< Display the Duty-cycle Selected For PWM_D
        }

        private void ExitAction(object sender, CancelEventArgs e)
        {
            DeInitGPIOLib();
            DeInitPWM(0);
            DeInitPWM(1);
            DeInitPWM(2);
            DeInitPWM(3);
        }
    }
}